/*
*  Name:      date.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type date
*  Date:      $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8538 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_DATE__
#define _UESQL_DATE__

#include <mpcl/text/local_date.hh>
#include <mpcl/text/string.hh>
#include "type.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  using std::string;

  ///  SQL date handler.
  class TDate                     :
    public mpcl::text::TLocalDate ,
    public IType
  {

    public:

      //
      //  C O N S T R U C T O R S
      //

      ///  Builds a new instance.
      TDate (void)    :
        TLocalDate () ,
        IType      () {}

      ///  Builds a new instance from another instance.
      TDate (const TDate& rktDATE) :
        TLocalDate (rktDATE)       ,
        IType      ()
      {
        setNull (rktDATE.isNull());
      }

      TDate (const EDateType keDATE_TYPE) :
        TLocalDate (keDATE_TYPE)          ,
        IType      ()
      {
        setNull (false);
      }

      TDate (const int kiYEAR, const int kiMONTH, const int kiDAY) :
        TLocalDate (kiYEAR, kiMONTH, kiDAY)                        ,
        IType      ()
      {
        setNull (false);
      }

      explicit TDate (const char* pkcDATE) :
        TLocalDate (eEmpty)                ,
        IType      ()
      {
        set (pkcDATE);
      }

      explicit TDate (const string& rkyDATE) :
        TLocalDate (eEmpty)                  ,
        IType      ()
      {
        set (rkyDATE);
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE);

      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      mpcl::text::TString get (void) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get ("'%Y-%m-%d'");
        }
        return yResult;
      }

      /**
      *  Returns date in SQL format.
      *  @param pkcDATE_FORMAT Date format in strftime-like format. It defaults
      *                        to ANSI SQL date  format.  A  valid date  can be
      *                        '1970-12-31'.
      *  @return Date in readable format.
      *  @warning pkcDATE_FORMAT must include the enclosing quotes (').
      */
      mpcl::text::TString get (const char* pkcDATE_FORMAT) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = TLocalDate::get (pkcDATE_FORMAT);
        }
        return yResult;
      }

  };  // class TDate

}  // namespace uesqlc


#endif  // not _UESQL_DATE__
