/*
*  Name:      identifier.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type identifier
*  Date:      $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8538 $
*
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_IDENTIFIER__
#define _UESQL_IDENTIFIER__

#include "string.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  ///  SQL type for SQL identifiers.
  class TIdentifier : public TString
  {

    public:
      
      //
      //  C O N S T R U C T O R S
      //
      
      explicit TIdentifier (void) : TString() {}

      ///  Builds a new instance from another instance.
      TIdentifier (const TIdentifier& rkyIDENTIFIER) :
        TString (rkyIDENTIFIER)                      {}

      TIdentifier ( const TIdentifier& rkyIDENTIFIER  ,
                    size_type          zOFFSET        ,
                    size_type          zLENGTH = npos ) :
        TString (rkyIDENTIFIER, zOFFSET, zLENGTH)       {}

      explicit TIdentifier (const mpcl::text::TString& rkySOURCE) : TString (rkySOURCE) {}

      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @param zLENGTH  Source string length.
      */
      TIdentifier (const char* pkcVALUE, size_type zLENGTH) :
        TString (pkcVALUE, zLENGTH)                         {}
      
      /**
      *  Extends the parent constructor with support for NULL strings.
      *  @param pkcVALUE Source string.
      *  @exception TExceptionEL_Constraint If source string is NULL.
      */
      explicit TIdentifier (const char* pkcVALUE) : TString (pkcVALUE) {}

      TIdentifier (size_type zTIMES, char cSOURCE) :
        TString (zTIMES, cSOURCE)                  {}
      
      template<typename InputIterator>
      TIdentifier (InputIterator tBEGIN_ITER, InputIterator tEND_ITER) :
        TString (tBEGIN_ITER, tEND_ITER)                               {}

      ///  Imports all versions of operator =.
      using TString::operator =;


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string without quotes.
      */
      mpcl::text::TString get (void) const
      {
        return ( isNull() ) ? mpcl::text::TString ("null") : (mpcl::text::TString) *this;
      }

  };  // class TIdentifier

}  // namespace uesqlc


#endif  // not _UESQL_IDENTIFIER__
