/*
*  Name:      real.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type real
*  Date:      $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8538 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_REAL__
#define _UESQL_REAL__

#include "type.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  ///  SQL type REAL.
  class TReal : public IType
  {

    private:

      ///  Real value.
      double   dValue;


    public:

      //
      //  C O N S T R U C T O R S
      //

      TReal (void) :
        IType  ()  ,
        dValue (0) {}

      ///  Builds a new instance from another instance.
      TReal (const TReal& rktREAL) :
        IType  ()                  ,
        dValue (rktREAL.dValue)
      {
        setNull (rktREAL.isNull());
      }

      TReal (const double kdVALUE) :
        IType  ()                  ,
        dValue (kdVALUE)
      {
        setNull (false);
      }

      explicit TReal (const char* pkcREAL) :
        IType  ()                          ,
        dValue (0)
      {
        set (pkcREAL);
      }

      explicit TReal (const std::string& rkyREAL) :
        IType  ()                                 ,
        dValue (0)
      {
        set (rkyREAL);
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null or points to a zero
      *  length string, then the instance is null.
      *  @pre ( !pkcVALUE || !*pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE)
      {
        setNull ( !pkcVALUE || !*pkcVALUE );
        if ( !isNull() )
        {
          dValue = std::atof (pkcVALUE);
        }
      }
      
      /**
      *  Assigns value \a rkyVALUE.  If \a rkyVALUE is empty, then the instance
      *  is null.
      *  @pre ( rkyVALUE.empty() ) => isNull()
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        set (rkyVALUE.c_str());
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string.
      */
      mpcl::text::TString get (void) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::text::Format ("%f", dValue);
        }
        return yResult;
      }

      /**
      *  Returns a string.
      *  @param pkcFORMAT Format string for converting from double to string.
      *  @return The result of converting this instance to string.
      *  @see std::printf(), mpcl::Format().
      */
      mpcl::text::TString get (const char* pkcFORMAT) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = mpcl::text::Format (pkcFORMAT, dValue);
        }
        return yResult;
      }

      ///  Returns the SQL real value.
      operator long double (void) const
      {
        return dValue;
      }

  };  // class TReal

}  // namespace uesqlc


#endif  // not _UESQL_REAL__
