/*
*  Name:      search_condition.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   UESQLC base SQL type identifier
*  Date:      $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8538 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _UESQL_SEARCH_CONDITION__
#define _UESQL_SEARCH_CONDITION__

#include <mpcl/text/string.hh>
#include "type.hh"


///  Name-space for UESQLC.
namespace uesqlc
{

  ///  SQL type for SQL search conditions.
  class TSearchCondition       :
    public IType               ,
    public mpcl::text::TString
  {

    public:

      ///  Import this type.
      typedef
        mpcl::text::TString::size_type
        size_type;


    public:

      //
      //  C O N S T R U C T O R S
      //

      explicit TSearchCondition (void) :
        IType               ()         ,
        mpcl::text::TString ()         {}

      ///  Builds a new instance from another instance.
      TSearchCondition (const TSearchCondition& rktSEARCH_CONDITION) :
        IType               ()                                       ,
        mpcl::text::TString (rktSEARCH_CONDITION)                    {}

      TSearchCondition& operator = (const TSearchCondition& rktSEARCH_CONDITION)
      {
        mpcl::text::TString::operator = (rktSEARCH_CONDITION);
        setNull (rktSEARCH_CONDITION.isNull());
        return *this;
      }

      /**
      *  Assigns value \a pkcVALUE.  If \a pkcVALUE is null,  then the instance
      *  is null.
      *  @pre ( !pkcVALUE ) => isNull()
      *  @param pkcVALUE Source value.
      */
      void set (const char* pkcVALUE)
      {
        if ( !pkcVALUE )
        {
          erase();
          setNull (true);
        }
        else
        {
          assign (pkcVALUE);
          setNull (false);
        }
      }

      /**
      *  Assigns value \a rkyVALUE.
      *  @param rkyVALUE Source value.
      */
      void set (const std::string& rkyVALUE)
      {
        assign (rkyVALUE);
        setNull (false);
      }


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Returns a string.
      *  @return The result of converting this instance to string without quotes.
      */
      mpcl::text::TString get (void) const
      {
        mpcl::text::TString   yResult;

        if ( isNull() )
        {
          yResult = "null";
        }
        else
        {
          yResult = *this;
        }
        return yResult;
      }

  };  // class TSearchCondition

}  // namespace uesqlc


#endif  // not _UESQL_SEARCH_CONDITION__
