/*
*  Name:         end.uesql.cc
*  Author:       Francisco Vides Fernandez
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      Destroy iterators test database
*  Date:         $Date: 2004-09-20 02:43:23 +0200 (lun, 20 sep 2004) $
*  Revision:     $Revision: 10141 $
*
*  Copyright (C) 2001-2003  Francisco Vides Fernandez <pax@dedaloingenieros.com>
*  Copyright (C) 2004       Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/
#include <iostream>
#include <mpcl/exceptions.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>

using uesqlc::TSqlerrorException;


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;


//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static void _DropTestCatalog (const mpcl::text::TString& rkyDBMS_SERVER_NAME)
{

  using uesqlc::ComposeCatalogLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TString;

  TString   yDbmsCatalogLocation;
  TString   yDbmsTestCatalogLocation;
  TString   yDbmsUserLogin;

  yDbmsCatalogLocation     = ComposeCatalogLocation (rkyDBMS_SERVER_NAME, "test");
  yDbmsTestCatalogLocation = ComposeCatalogLocation (rkyDBMS_SERVER_NAME, "TEST_ITERATORS_CATALOG");
  yDbmsUserLogin           = ComposeUserLogin ("test", "test");
  TEST_WRITE_INFO ("DBMS-CATALOG-LOCATION      = %s", yDbmsCatalogLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN            = %s", yDbmsUserLogin.c_str());
  TEST_WRITE_INFO ("DBMS-TEST-CATALOG-LOCATION = %s", yDbmsTestCatalogLocation.c_str());
  extern "UESQL::default"
  {
    --
    --  Connects to default catalog.
    --
    connect to :yDbmsCatalogLocation user :yDbmsUserLogin;
    --
    --  Removes catalog test_iterators_catalog.
    --
    drop catalog test_iterators_catalog cascade;
  }

}   // _DropTestCatalog()


int main (int argc, const char** argv)
{

  mpcl::util::prefs::TConfigProcessor   tConfigProcessor (argc, argv);
  const char*                           pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  TEST_INIT ("Cleans the DBMS from test catalogs");
  TEST_START_WATCH;
  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }

    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor["dbms-server-name"].setUsageString ("Server where database resides");
    tConfigProcessor.processOptions();
    
    if ( !TEST_IN_SILENT_MODE )
    {
      mpcl::util::logging::tLog.also (std::cerr);
    }
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition throw an exception
      --
      whenever sqlerror throw TSqlerrorException;
      
    }
    _DropTestCatalog (tConfigProcessor ["dbms-server-name"].getValue());
  }
  catch (const uesqlc::TSqlerrorException& rktEXCEPTION)
  {
    std::cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << std::endl;
    std::cerr << "SQLCODE="     << uesqlc::SQLCODE     << std::endl;
    std::cerr << "DESCRIPTION=" << _GetErrorMessage() << std::endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    std::cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }  
  catch (...)
  {
    std::cerr << "Unhandled exception" << std::endl;
    TEST_FAIL_WITH_CODE (3);
  }
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (std::cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
