/*
*  Name:         test-001-sql_types.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2004-09-20 02:43:23 +0200 (lun, 20 sep 2004) $
*  Revision:     $Revision: 10141 $
*
*  Copyright (C) 1996-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesqlc::TDate;
  using uesqlc::TInteger;
  using uesqlc::TReal;
  using uesqlc::TString;
  using uesqlc::TTime;

  TDate      tDate;
  TInteger   tInteger;
  TReal      tReal;
  TTime      tTime;
  TString    yString;
  TString    yString4;
  TString    yString1;
  TString    yString2;
  TString    yString3;
  int        iRecordCount = 0;

  //
  //  Checks for quote escaping.
  //
  TEST_STRINGS ("'hello'",        TString ("hello").get().c_str());
  TEST_STRINGS ("'hello''bye'",   TString ("hello'bye").get().c_str());
  TEST_STRINGS ("'hello''''bye'", TString ("hello''bye").get().c_str());
  TEST_STRINGS ("'hello\\\\bye'", TString ("hello\\bye").get().c_str());

  //
  //  Other checks.
  //
                                     TEST_NUMBERS (1,     TInteger (true));
                                     TEST_NUMBERS (0,     TInteger (false));
  tInteger.set ("1");                TEST_NUMBERS (1,     tInteger);
  tInteger.set ("0");                TEST_NUMBERS (0,     tInteger);
  tInteger.set ("10000");            TEST_NUMBERS (10000, tInteger);
                                     TEST_NUMBERS (10001, TInteger (10001));
  tDate.set    ((const char*) NULL); TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ((const char*) NULL); TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ((const char*) NULL); TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ((const char*) NULL); TEST_NUMBERS (true,  yString.isNull());
  tDate.set    ("");                 TEST_NUMBERS (true,  tDate.isNull());
  tInteger.set ("");                 TEST_NUMBERS (true,  tInteger.isNull());
  tReal.set    ("");                 TEST_NUMBERS (true,  tReal.isNull());
  yString.set  ("");                 TEST_NUMBERS (false, yString.isNull());
                                     TEST_NUMBERS (true,  TDate().isNull());
                                     TEST_NUMBERS (true,  TInteger().isNull());
                                     TEST_NUMBERS (true,  TReal().isNull());
                                     TEST_NUMBERS (true,  TString().isNull());
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.getBuffer (12);
  tInteger.set ("75000");
  TEST_NUMBERS (75000, tInteger);
  extern "UESQL::default" 
  {
    whenever sqlerror  goto error;
    whenever not found goto loop_end;
    create table table_1
    (
      code       character (3) not null ,
      birth_date date                   ,
      birth_time time                   ,
      salary     integer                ,
      percentage double precision
    );
    insert into table_1 values ('000', '1999-12-31', '01:00', 12000, 25.75);
    insert into table_1 values ('001', '1970-01-01', '02:00', 75000, 30);
    insert into table_1 values ('002', '2000-01-01', '03:00', 11500, 99.99);
    insert into table_1 values ( '003'                 ,
                                 :TDate (2000, 12, 31) ,
                                 :TTime ("12:00:00")   ,
                                 :TInteger (7500)      ,
                                 :TReal (33.33)        );
    insert into table_1 values ( '004'                 ,
                                 :TDate (2001, 1, 1) ,
                                 :TTime (23, 59, 59) ,
                                 :TInteger (6500)    ,
                                 :TReal (0.0)        );
    --
    --  Trick to get one single row for escape testing.
    --
    select 'hello\rbye'    into :yString1 from table_1 where ( code = '000' );
    select 'hello"bye'     into :yString2 from table_1 where ( code = '000' );
    select 'hello''bye'    into :yString3 from table_1 where ( code = '000' );
    select :TString ("\\") into :yString4 from table_1 where ( code = '000' );

    --
    --  Cursor testing.
    --
    declare mycursor cursor for
      select code, birth_date, birth_time, salary, percentage from table_1;
    open mycursor;
  }
  TEST_STRINGS ("'hello\rbye'", yString1.get().c_str());
  TEST_STRINGS ("'hello\"bye'", yString2.get().c_str());
  TEST_STRINGS ("'hello''bye'", yString3.get().c_str());
  TEST_STRINGS ("'\\\\'",       yString4.get().c_str());
  TEST_NUMBERS ("hello\rbye",   yString1);
  TEST_NUMBERS ("hello\"bye",   yString2);
  TEST_NUMBERS ("hello'bye",    yString3);
  TEST_NUMBERS ("\\",           yString4);

  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE  BIRTH_DATE   BIRTH_DATE SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "===== ============ ========== ====== ==========");
  }
  while ( true )
  {
    extern "UESQL::default"
    {
      fetch from mycursor into :yString, :tDate, :tTime, :tInteger, :tReal;
    }
    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile              ,
                "%5s %12s %10s %6ld %10.03f\n" ,
                yString.get().c_str()          ,
                tDate.get().c_str()            ,
                tTime.get().c_str()            ,
                (long int) tInteger            ,
                (double) tReal                 );
    }
    ++iRecordCount;
  }

loop_end:
  extern "UESQL::default" 
  {
    close mycursor;
    drop table table_1 cascade;
  }
  if ( iRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the catalog location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeCatalogLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TIdentifier;
  using uesqlc::TInteger;
  using uesqlc::TSearchCondition;
  using uesqlc::TString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TInteger           tHours;
  TInteger           tCount;
  TSearchCondition   tCondition;
  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TIdentifier        yCursorName;
  TString            yDbmsCatalogLocation;
  TString            yDbmsUserLogin;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TString            yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName      = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin       = ComposeUserLogin       ("test",          "test");
    yDbmsCatalogLocation = ComposeCatalogLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-CATALOG-LOCATION = %s", yDbmsCatalogLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN       = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  WHENEVER
      --
      whenever sqlerror goto error;
      
      --
      --  CONNECT TO
      --
      connect to :yDbmsCatalogLocation user :yDbmsUserLogin;
    }
    TEST_FUNCTION (_TestSqlTypes());
    extern "UESQL::default" 
    {
      --
      --  DISCONNECT
      --
      disconnect current;
    }
    goto end;

  error:
    cerr << "SQLSTATE="    << uesqlc::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesqlc::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
