/*
*  Name:      test-002-catalog_creation.uesql.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for UESQL
*  Date:      $Date: 2004-09-20 02:43:23 +0200 (lun, 20 sep 2004) $
*  Revision:  $Revision: 10141 $
*
*  Copyright (C) 2000-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/system/all.hh>
#include <mpcl/test.h>
#include <mpcl/text/string.hh>
#include <mpcl/util/logging/log.hh>
#include <mpcl/util/prefs/config_processor.hh>


//
//  L O C A L   V A R I A B L E S
//

static uesqlc::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::text::TString _GetErrorMessage (void)
{

  using mpcl::text::TString;

  return ( TString (_tSqlstateDescriptor.getClassValue())    +
           ": "                                              +
           TString (_tSqlstateDescriptor.getSubclassValue()) +
           " (" + uesqlc::SQLMESSAGE + ")"                    );

}  // _GetErrorMessage()


/**
*  Entry  point.  It uses the  user login 'test/test'  and the catalog location
*  'localhost/test'.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*/
int main (int argc, const char* argv[])
{

  using mpcl::text::Format;
  using mpcl::util::prefs::TConfigProcessor;
  using std::cerr;
  using std::endl;
  using uesqlc::ComposeCatalogLocation;
  using uesqlc::ComposeUserLogin;
  using uesqlc::TString;
  
  TEST_INIT ("Test for CATALOG related statements");
  TEST_START_WATCH;

  TString            yVar0;
  TString            yVar1;
  TString            yVar2;
  TString            yCursorName;
  TString            yDbmsCatalogLocation;
  TString            yDbmsUserLogin;
  TString            yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name", "o", pcDbmsServerName);
    tConfigProcessor.processOptions();
    yDbmsServerName      = tConfigProcessor ["dbms-server-name"].getValue();
    yDbmsUserLogin       = ComposeUserLogin       ("test",          "test");
    yDbmsCatalogLocation = ComposeCatalogLocation (yDbmsServerName, "test");
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-CATALOG-LOCATION = %s", yDbmsCatalogLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN       = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.also (cerr);
  }
  if ( !yDbmsServerName.empty() )
  {
    extern "UESQL::default" 
    {
      --
      --  Whenever any SQLERROR condition goto label 'error'.
      --
      whenever sqlerror goto error;

      --
      --  Connects to default catalog.
      --
      connect to :yDbmsServerName user :yDbmsUserLogin;

      --
      --  Creates catalog 'test_002_catalog'.
      --
      create catalog test_002_catalog;
      set catalog 'test_002_catalog';
      create table table_1
      (
        code         character (3) not null ,
        worker_count integer
      );
      create table table_2
      (
        work_code character (3)  not null ,
        name      character (40) not null ,
        hours     decimal   (5)
      );

      --
      --  Connects to the newly created catalog.
      --
      connect to :ComposeCatalogLocation (yDbmsServerName, "test_002_catalog")
        as 'myconnection'
        user :yDbmsUserLogin;

      --
      --  Removes tables.
      --
      drop table table_1 cascade;
      disconnect current;
      set connection default;
    }

    //
    //  Sleeps for 5  seconds to  let slow  systems (like POSTGRESQL on a i586)
    //  remove connections properly.
    //
    mpcl::system::tSystem.sleep (5);
    extern "UESQL::default" 
    {
      --
      --  Removes catalog 'test_002_catalog'.
      --
      set catalog 'test';
      drop catalog test_002_catalog cascade;
    }
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesqlc::SQLSTATE   << endl;
  cerr << "SQLCODE="     << uesqlc::SQLCODE    << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesqlc::SQLCODE);

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::util::logging::tLog.erase (cerr);
  }
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
