/*
*  Name:         uesqlc.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Summary:      UESQLC main
*  Date:         $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8538 $
*  Description:  Compiler for UESQL language.
*                UESQL is a subset of:
*
*                ISO/IEC 9075:1992, "Information Technology -- Database Languages -- SQL"
*                ANSI X3.135-1992,  "Database Language SQL"
*
*  Copyright (C) 1996-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <clocale>
#include "application.hh"


//
//  E X P O R T E D   V A R I A B L E S
//

///  Compiler application pointer.
TApplication*   ptApplication;


///  Main.
int main (int argc, const char* argv[])
{

  using std::cerr;
  using std::endl;

  int   iExitCode;
  
  std::setlocale (LC_ALL, "");
  try
  {
    ptApplication = new TApplication (PACKAGE, VERSION, argc, argv);
    ptApplication->initialize();
    iExitCode = ptApplication->start();
    delete ptApplication;
  }
  catch (const mpcl::util::prefs::TNotParameterAbleException& rktEXCEPTION)
  {
    //
    //  Exception:  Not parameter-able option.
    //
    iExitCode = 4;
    cerr << rktEXCEPTION.generalDescription();
    if ( !rktEXCEPTION.specificDescription().empty() )
    {
      cerr << " (--" << rktEXCEPTION.specificDescription() << ")";
    }
    cerr << endl;
    delete ptApplication;
  }
  catch (const mpcl::TException& rktEXCEPTION)
  {
    iExitCode = 3;
    cerr << rktEXCEPTION;
    delete ptApplication;
  }
  catch (const std::bad_alloc& rktEXCEPTION)
  {
    iExitCode = 2;
    cerr << rktEXCEPTION.what() << endl;
  }
  catch (const std::exception& rktEXCEPTION)
  {
    iExitCode = 2;
    cerr << rktEXCEPTION.what() << endl;
    delete ptApplication;
  }
  catch (...)
  {
    iExitCode = 1;
    cerr << mpcl::TException ("unhandled exception", NULL, __FILE__, __LINE__);
    delete ptApplication;
  }
  return iExitCode;
  
}  // main()
