/*
*  Name:      TInteger.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type integer.
*  Date:      $Date: 2003-11-27 12:34:16 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8538 $
*
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;


///  SQL type for integer numbers.
public class TInteger implements IType
{

  ///  Value.
  private Long   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TInteger() {}

  /**
  *  Builds a new instance.
  *  @param kgVALUE Source boolean.
  *  @post ( !isNull() ) && ( intValue() == (( kgVALUE ) ? 1 : 0) )
  */
  public TInteger (final boolean kgVALUE)
  {
    tValue = new Long (( kgVALUE ) ? 1 : 0);
  }

  /**
  *  Builds a new instance.
  *  @param kiVALUE Source integer.
  *  @post ( !isNull() )
  */
  public TInteger (final long kiVALUE)
  {
    tValue = new Long (kiVALUE);
  }

  /**
  *  Builds a new instance.
  *  @param ktVALUE Source integer.
  *  @post ( !isNull() )
  */
  public TInteger (final Integer ktVALUE)
  {
    if ( ktVALUE != null )
    {
      tValue = new Long (ktVALUE.longValue());
    }
  }

  /**
  *  Builds a new instance.
  *  @param ktVALUE Source short.
  *  @post ( !isNull() )
  */
  public TInteger (final Short ktVALUE)
  {
    if ( ktVALUE != null )
    {
      tValue = new Long (ktVALUE.longValue());
    }
  }

  /**
  *  Builds a new instance.
  *  @param kyINTEGER Source integer.
  *  @post ( isNull() == ( null == kyINTEGER ) )
  */
  public TInteger (final String kyINTEGER)
  {
    if ( kyINTEGER != null )
    {
      tValue = new Long (kyINTEGER);
    }
  }

  public IType set (final Object ktVALUE)
  {

    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof Integer )
    {
      tValue = new Long (((Integer) ktVALUE).longValue());
    }
    else if ( ktVALUE instanceof Long )
    {
      tValue = new Long (((Long) ktVALUE).longValue());
    }
    else if ( ktVALUE instanceof Short )
    {
      tValue = new Long (((Short) ktVALUE).longValue());
    }
    else if ( ktVALUE instanceof String )
    {
      tValue = new Long ((String) ktVALUE);
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;

  }  // set()


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns a new instance built from \a kiSOURCE.
  *  @param kiSOURCE Source integer.
  *  @return A new instance.
  */
  public static TInteger _valueOf (final int kiSOURCE)
  {
    return new TInteger (kiSOURCE);
  }

  /**
  *  Returns a new instance built from \a kgSOURCE.
  *  @param kgSOURCE Source boolean.
  *  @post ( kgSOURCE ) ? ( intValue() == 1 ) : ( intValue() == 0 )
  *  @return A new instance.
  */
  public static TInteger _valueOf (final boolean kgSOURCE)
  {
    return new TInteger (kgSOURCE);
  }

  /**
  *  Returns a new instance built from \a ktSOURCE.
  *  @param ktSOURCE Source integer.
  *  @return A new instance.
  */
  public static TInteger _valueOf (final Integer ktSOURCE)
  {
    return new TInteger (ktSOURCE);
  }

  /**
  *  Returns a new instance built from \a ktSOURCE.
  *  @param ktSOURCE Source short.
  *  @return A new instance.
  */
  public static TInteger _valueOf (final Short ktSOURCE)
  {
    return new TInteger (ktSOURCE);
  }

  public String get()
  {

    String   yResult = "";

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = tValue.toString();
    }
    return yResult;

  }  // get()

  /**
  *  Returns true if the instance holds a value different from zero.
  *  @return True if different from zero.
  */
  public boolean booleanValue()
  {
    return ( tValue.intValue() != 0 );
  }

  public int intValue()
  {
    return tValue.intValue();
  }

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public long longValue()
  {
    return tValue.longValue();
  }

  public Integer toInteger()
  {
    return ( tValue == null ) ? null : new Integer (tValue.intValue());
  }

  public String toString()
  {
    return ( tValue == null ) ? null : tValue.toString();
  }

}  // class TInteger
