/*
*  Name:      TTime.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type time.
*  Date:      $Date: 2004-11-10 02:04:03 +0100 (mié, 10 nov 2004) $
*  Revision:  $Revision: 10417 $
*
*  Copyright (C) 2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.sql.Time;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;


///  SQL type for time.
public class TTime implements IType
{

  private Time   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TTime() {}

  /**
  *  Builds a new instance.
  *  @param ktTIME Source time.
  *  @post ( isNull() == ( null == ktTIME ) )
  */
  public TTime (final Time ktTIME)
  {
    if ( ktTIME != null )
    {
      tValue = (Time) ktTIME.clone();
    }
  }

  /**
  *  Builds a new instance.
  *  @param kiHOUR   Source hour (0-23).
  *  @param kiMINUTE Source minute (0-59).
  *  @param kiSECOND Source second (0-59).
  *  @post ( !isNull() )
  */
  public TTime (final int kiHOUR, final int kiMINUTE, final int kiSECOND)
  {
    GregorianCalendar   tGregorianCalendar;

    tGregorianCalendar = new GregorianCalendar();
    tGregorianCalendar.set (Calendar.HOUR_OF_DAY, kiHOUR);
    tGregorianCalendar.set (Calendar.MINUTE,      kiMINUTE);
    tGregorianCalendar.set (Calendar.SECOND,      kiSECOND);
    tValue = new Time (tGregorianCalendar.getTimeInMillis());
  }

  /**
  *  Builds a new instance.
  *  @param kyTIME Source time.
  *  @post ( isNull() == ( null == kyTIME ) )
  */
  public TTime (final String kyTIME)
  {
    if ( kyTIME != null )
    {
      tValue = Time.valueOf (kyTIME);
    }
  }

  public IType set (final Object ktVALUE)
  {

    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof Time )
    {
      tValue = (Time) ((Time) ktVALUE).clone();
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;

  }  // set()


  //
  //  S E L E C T O R S
  //

  public String get()
  {

    String   yResult;

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = "'" + tValue + "'";
    }
    return yResult;

  }  // get()

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public Date toDate()
  {
    return ( tValue == null ) ? null : new Date (tValue.getTime());
  }

  public String toString()
  {
    return ( tValue == null ) ? null : tValue.toString();
  }

}  // class TTime
