/*
*  Name:      TTimestamp.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   SQL type timestamp.
*  Date:      $Date: 2004-11-13 02:22:31 +0100 (sáb, 13 nov 2004) $
*  Revision:  $Revision: 10437 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package for UESQL.
package org.uesqlc;

import java.sql.Timestamp;
import java.util.Date;


///  SQL type for timestamps.
public class TTimestamp implements IType
{

  private Timestamp   tValue = null;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( isNull() )
  */
  public TTimestamp() {}

  /**
  *  Builds a new instance.
  *  @param ktTIMESTAMP Source time.
  *  @post ( isNull() == ( null == ktTIMESTAMP ) )
  */
  public TTimestamp (final long klTIME)
  {
    tValue = new Timestamp (klTIME);
  }

  /**
  *  Builds a new instance.
  *  @param ktDATE Source date.
  *  @post ( isNull() == ( null == ktDATE ) )
  */
  public TTimestamp (final Date ktDATE)
  {
    if ( ktDATE != null )
    {
      tValue = new Timestamp (ktDATE.getTime());
    }
  }

  /**
  *  Builds a new instance.
  *  @param ktTIMESTAMP Source time.
  *  @post ( isNull() == ( null == ktTIMESTAMP ) )
  */
  public TTimestamp (final Timestamp ktTIMESTAMP)
  {
    if ( ktTIMESTAMP != null )
    {
      tValue = (Timestamp) ktTIMESTAMP.clone();
    }
  }

  public IType set (final Object ktVALUE)
  {

    if ( ktVALUE == null )
    {
      tValue = null;
    }
    else if ( ktVALUE instanceof Timestamp )
    {
      tValue = (Timestamp) ((Timestamp) ktVALUE).clone();
    }
    else
    {
      throw new IllegalArgumentException ("invalid instance of " + ktVALUE.getClass());
    }
    return this;

  }  // set()


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns a new instance built from \a kiSOURCE.
  *  @param kiSOURCE Source integer.
  *  @return A new instance.
  */
  public static TTimestamp _valueOf (final Date ktSOURCE)
  {
    return new TTimestamp (ktSOURCE);
  }

  public String get()
  {

    String   yResult;

    if ( isNull() )
    {
      yResult = "null";
    }
    else
    {
      yResult = "'" + tValue + "'";
    }
    return yResult;

  }  // get()

  public boolean isNull()
  {
    return ( tValue == null );
  }

  public Date toDate()
  {
    return ( tValue == null ) ? null : new Date (tValue.getTime());
  }

  public String toString()
  {
    return ( tValue == null ) ? null : tValue.toString();
  }

}  // class TTimestamp
